<?php
namespace SetformsDev\Util {

abstract class Entity extends \Setforms2\Shared\ORM\BasicEntity
{

    CONST YES = 'Y';
    CONST NO = 'N';

    public function __construct() {
        $this->setPkValues();
        $this->setIsactive(self::YES);
        $this->setTkOrganizationId(1);
        $this->setAuditValues();
    }


    private function setAuditValues() {
        $this->setCreated(new \Datetime()); //now()
        $this->setCreatedBy(self::getCurrentUser());
        $this->updateUpdateFields();
    }
    
    private function setPkValues() {
        $reflacton = new \ReflectionClass($this);
        $class_name = $reflacton->getShortName();
        $property_name = lcfirst($class_name."Id");

        $next_value = Sequence::getNextValue(Inflector::tableize($class_name));

        // Entity properties should be inaccessible.
        $reflection_property = new \ReflectionProperty($this, $property_name);
        $reflection_property->setAccessible(true);
        $reflection_property->setValue($this, $next_value);
        $reflection_property->setAccessible(false);
    }


    public function updateUpdateFields() {
        $this->setUpdated(new \Datetime()); //now()
        $this->setUpdatedBy(self::getCurrentUser());
    }


    protected static function getCurrentUser() {
        return \SetformsDev\Entities\TkUser::getObject(tk_get_env_var('TK_USER_ID'));
    }


    public function persist()
    {
        $this->updateUpdateFields();
        parent::persist();
    }

    /**
     * Duplicate a entity object.
     * 
     * IMPORTANT: If a specific property can't be duplicated, you need treat this manually.
     * IMPORTANTE: Se uma propriedade específica não pode ser duplicada, você precisa tratar isto manualmente.
     * 
     * @return \SetformsDev\Util\Entity 
     */
    public function duplicate() {
        $entity = clone $this;
        $entity->setPkValues();
        $entity->setAuditValues();
        return $entity;
    }

}
}